package Net::Amazon::EMR::AddJobFlowStepsResult;
use Moose;
with 'Net::Amazon::EMR::Role::AttrHash';
use Net::Amazon::EMR::Coercions;


has 'StepIds' => ( is => 'ro',
                   isa => 'Net::Amazon::EMR::Type::ArrayRefofConstrainedStr',
                   coerce => 1 );


__PACKAGE__->meta->make_immutable();

1;

__END__

=head1 NAME

Net::Amazon::EMR::AddJobFlowStepsResult

=head1 DESCRIPTION

Implements the AddJobFlowSteps result type described at L<http://docs.amazonwebservices.com/ElasticMapReduce/latest/API/API_AddJobFlowSteps.html>.

=head1 AUTHOR

Jon Schutz 

L<http://notes.jschutz.net>

=head1 DOCUMENTATION, LICENSE AND COPYRIGHT

See L<Net::Amazon::EMR>.

=cut
